from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.views.decorators.http import require_POST
from .oss import upload_to_oss
import uuid
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt


@method_decorator(csrf_exempt, name="dispatch")
@require_POST
@login_required
def upload_view(request):
    try:
        image_file = request.FILES["image"]
        user_id = request.user.id

        # 初始化pathname变量
        if isinstance(user_id, uuid.UUID):
            pathname = f"user-{user_id}"
        else:
            # 将数字 ID 格式化为 8 位带前导零
            try:
                pathname = f"user-{int(user_id):08d}"
            except ValueError:
                # 如果既不是 UUID 也不是数字，保留原始值
                pathname = f"user-{str(user_id)}"

        print(pathname)
        oss_url = upload_to_oss(image_file, pathname)
        return JsonResponse({"url": oss_url})
    except Exception as e:
        return JsonResponse({"error": str(e)}, status=400)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
import oss2


def get_oss_config():
    return getattr(
        settings,
        "DJ_IMAGE_UPLOADER_OSS_CONFIG",
    )


def upload_to_oss(file_obj, pathname=None):
    config = get_oss_config()
    if pathname:
        filename = f"{pathname}/{file_obj.name}"
    else:
        filename = f"{config['BASE_PATH']}/{file_obj.name}"
    try:
        auth = oss2.Auth(config["ACCESS_KEY_ID"], config["ACCESS_KEY_SECRET"])
        bucket = oss2.Bucket(auth, config["ENDPOINT"], config["BUCKET_NAME"])

        filename = f"{pathname}/{file_obj.name}"
        bucket.put_object(filename, file_obj)
        return f"https://{config['BUCKET_NAME']}.{config['ENDPOINT']}/{filename}"
    except KeyError as e:
        raise ImproperlyConfigured(f"Missing OSS config: {e}")



from django.db import models
from .widgets import ImageUploaderWidget
from django import forms


class ImageUploaderField(models.URLField):
    def __init__(self, upload_url=None, *args, **kwargs):
        self.upload_url = upload_url
        super().__init__(*args, **kwargs)

    @staticmethod
    def _get_form_class():
        return ImageUploaderFormField

    def formfield(self, **kwargs):
        defaults = {
            "form_class": ImageUploaderFormField,
            "widget": ImageUploaderWidget(upload_url=self.upload_url),
            "upload_url": self.upload_url,
        }
        defaults.update(kwargs)
        return super(ImageUploaderField, self).formfield(**defaults)


class ImageUploaderFormField(forms.fields.URLField):
    def __init__(self, upload_url=None, **kwargs):
        kwargs.update({"widget": ImageUploaderWidget(upload_url=upload_url)})
        super(ImageUploaderFormField, self).__init__(**kwargs)
